"use strict";
var udev = require("udev");
var InputEvent = require("input-event");
var strongly_typed_events_1 = require("strongly-typed-events");
var AVRCPController = /** @class */ (function () {
    function AVRCPController(context) {
        this.keyPressEvent = new strongly_typed_events_1.SimpleEventDispatcher();
        this.context = context;
        this.logger = this.context.logger;
        this.udevMonitor = udev.monitor();
        this.devicelist = {};
        this.udevInit();
    }
    AVRCPController.prototype.udevInit = function () {
        var self = this;
        self.udevMonitor.on('add', function (device) {
            self.logger.info('added device ' + JSON.stringify(device, null, 4));
            if (device.DEVPATH.startsWith('/devices/virtual/input/')) {
                if (device.DEVNAME && device.DEVNAME.startsWith('/dev/input/')) {
                    self.handleInputEvent(device);
                }
                else {
                    self.handleInputDevice(device);
                }
            }
        });
        self.udevMonitor.on('remove', function (device) {
            self.logger.info('AVRCP: removed device ' + JSON.stringify(device, null, 4));
            if (device.DEVNAME) {
                for (var mac in self.devicelist) {
                    if (self.devicelist[mac].devicePath == device.DEVNAME) {
                        self.devicelist[mac].keyboard.close();
                        delete self.devicelist[mac].keyboard;
                        self.devicelist[mac].inputEvent.close();
                        delete self.devicelist[mac].inputEvent;
                        self.devicelist[mac].devicePath = "";
                        self.devicelist[mac].virtualPath = "";
                        self.logger.info('AVRCP: removed device ' + mac + ' successfuly');
                    }
                }
            }
        });
    };
    AVRCPController.prototype.handleInputDevice = function (device) {
        var self = this;
        var stripedName = device.NAME.replace(/['"]+/g, '');
        self.logger.info('AVRCP: found new input device from MAC ' + stripedName);
        if (self.devicelist[stripedName]) {
            self.logger.info('AVRCP: accepted input device');
            self.devicelist[stripedName].virtualPath = device.DEVPATH;
        }
        else {
            self.logger.info('AVRCP: not accepted input device');
        }
    };
    AVRCPController.prototype.handleInputEvent = function (device) {
        var self = this;
        self.logger.info('AVRCP: new input event found :' + device.DEVPATH);
        for (var mac in self.devicelist) {
            if (device.DEVPATH.startsWith(self.devicelist[mac].virtualPath)) {
                self.logger.info('AVRCP: accepted input event');
                var item = self.devicelist[mac];
                item.devicePath = device.DEVNAME;
                self.logger.info('AVRCP: added input event');
                item.inputEvent = new InputEvent(item.devicePath);
                item.inputEvent.on('error', function (err) { });
                self.logger.info('AVRCP: created input event');
                item.keyboard = new InputEvent.Keyboard(item.inputEvent);
                self.logger.info('AVRCP: created keyboard');
                self.logger.info('AVRCP: register keyboard for input ' + item.keyboard);
                item.keyboard.on('keypress', function (key) {
                    self.logger.info('AVRCP detected keypress : ' + key);
                    self.keyPressEvent.dispatch(key);
                });
                item.keyboard.fd.on('error', function (err) { });
                return;
            }
        }
        self.logger.info('AVRCP: not accepdted input event found');
    };
    Object.defineProperty(AVRCPController.prototype, "onKeyPress", {
        get: function () {
            return this.keyPressEvent.asEvent();
        },
        enumerable: true,
        configurable: true
    });
    AVRCPController.prototype.addDevice = function (mac) {
        var self = this;
        self.logger.info('AVRCP: adding device: ' + mac);
        var known = self.devicelist[mac];
        if (!known) {
            self.devicelist[mac] = { virtualPath: '', devicePath: '', inputEvent: null, keyboard: null };
        }
        self.logger.info('known: ' + JSON.stringify(self.devicelist, null, 4));
    };
    AVRCPController.prototype.removeDevice = function (mac) {
        var self = this;
        var known = self.devicelist[mac];
        if (known) {
            self.devicelist[mac].inputEvent.close();
            delete self.devicelist[mac];
        }
    };
    return AVRCPController;
}());
module.exports = AVRCPController;
//# sourceMappingURL=AVRCP.js.map
